"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const utils_1 = require("../utils");
const combine_objects_1 = require("./combine-objects");
function proxyCombineObjects(...objs) {
    // cache Object.keys() result
    let cachedOwnKeys;
    // cache 100 nonexistent keys that were accessed
    let nonexistentKeys;
    const handler = {
        get(memo, key) {
            if (utils_1.hasOwnProperty.call(memo, key)) {
                return memo[key];
            }
            if (nonexistentKeys && nonexistentKeys.has(key)) {
                return;
            }
            for (let i = objs.length - 1; i >= 0; i--) {
                const obj = objs[i];
                if (utils_1.hasOwnProperty.call(obj, key)) {
                    return (memo[key] = obj[key]);
                }
            }
            if (!nonexistentKeys) {
                nonexistentKeys = new Set();
            }
            else if (nonexistentKeys.size >= 100) {
                nonexistentKeys.clear();
            }
            nonexistentKeys.add(key);
        },
        ownKeys() {
            if (!cachedOwnKeys) {
                cachedOwnKeys = new Set(objs.reduce((arr, obj) => arr.concat(Object.keys(obj)), []));
            }
            return [...cachedOwnKeys];
        },
        getOwnPropertyDescriptor(memo, key) {
            if (utils_1.hasOwnProperty.call(memo, key)) {
                return { value: memo[key], configurable: true, enumerable: true };
            }
            if (nonexistentKeys && nonexistentKeys.has(key)) {
                return;
            }
            for (let i = objs.length - 1; i >= 0; i--) {
                const obj = objs[i];
                if (utils_1.hasOwnProperty.call(obj, key)) {
                    memo[key] = obj[key];
                    return { value: memo[key], configurable: true, enumerable: true };
                }
            }
            if (!nonexistentKeys) {
                nonexistentKeys = new Set();
            }
            else if (nonexistentKeys.size >= 100) {
                nonexistentKeys.clear();
            }
            nonexistentKeys.add(key);
        }
    };
    try {
        return new Proxy(Object.create(null), handler);
    }
    catch (e) {
        return combine_objects_1.combineObjects(...objs);
    }
}
exports.proxyCombineObjects = proxyCombineObjects;
//# sourceMappingURL=proxy-combine-objects.js.map