import { WrapResultInHandlers, DuplicatedKeysInObjects } from './types';
/**
 * Same as [[combineUniqueObjects]]:
 *
 * - Always returns a new object.
 * - The combined ActionHandlers won't lose index signature.
 * - Duplicated keys are not allowed.
 *   The resulted type will be inferred as `never`
 *   if there are(or potential) duplicated keys.
 *
 * Since it's wrapped with Proxy:
 *
 * - Properties are only created and cached on first visit.
 * - Fallback to [[combineUniqueObjects]] if `Proxy` is not supported.
 */
export declare function proxyCombineUniqueObjects<TObjects extends any[]>(...objs: TObjects): DuplicatedKeysInObjects<TObjects> extends never ? WrapResultInHandlers<TObjects[number]> : never;
