export declare const hasOwnProperty: (v: string | number | symbol) => boolean;
export declare type IntersectionFromUnion<TUnion> = (TUnion extends any ? (arg: TUnion) => void : never) extends (arg: infer TArg) => void ? TArg : never;
export interface DefaultActionCatalog {
    [type: string]: {
        payload?: any;
        meta?: any;
        error?: any;
    };
}
export interface DefaultAction {
    type: string;
}
export declare type DefaultActionCreator = (...args: any) => DefaultAction;
export declare type DefaultActionCreators = {
    readonly [type: string]: DefaultActionCreator;
};
export declare type CreateDefaultActionCreator = (type: string) => DefaultActionCreator;
export declare type MixedActionCreator = (...args: any) => any;
export declare type MixedActionCreators = {
    readonly [type: string]: MixedActionCreator;
};
export declare type CreateMixedActionCreator = (type: string) => MixedActionCreator;
/**
 * Basic actions configuration.
 * Index as action type.
 * Error type is added by `Action`.
 *
 * @template TCatalog ActionCatalog
 */
export declare type CreateActionCatalog<TCatalog extends DefaultActionCatalog> = TCatalog;
/**
 * Extract action types from a ActionCatalog
 *
 * @template TCatalog ActionCatalog
 */
export declare type ActionType<TCatalog> = Extract<keyof TCatalog, string>;
/**
 * Get action handler type of a module.
 * @template TState State.
 * @template TAction Action.
 */
export declare type DefaultActionHandler<TState extends {}, TAction extends DefaultAction> = (state: Readonly<TState>, action: TAction) => Readonly<TState>;
/**
 * Get all action handler types of a module.
 * @template TState Module state.
 */
export declare type DefaultActionHandlers<TState extends {}> = {
    readonly [key: string]: DefaultActionHandler<TState, DefaultAction>;
};
